#  Copyright (C) 2007 by
#  Davide Lo Re <boyska@gmail.com>
#  Sergio Pistone <sergio_pistone@yahoo.com.ar>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require "utils/strings"
require "lyrics"

require "cgi"

class LyricsMania < Lyrics

	def LyricsMania.site_host()
		return "www.lyricsmania.com"
	end

	def LyricsMania.site_name()
		return "LyricsMania"
	end

	def LyricsMania.build_song_add_url( artist, title, album=nil, year=nil )
		return "http://#{site_host()}/add.html"
	end

	def LyricsMania.build_google_feeling_lucky_url( artist, title=nil )
		query =  Strings.google_search_quote( artist )
		query << " " << Strings.google_search_quote( title + " lyrics" ) if ( title )
		return Strings.build_google_feeling_lucky_url( query, site_host() )
	end

	def build_lyrics_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => build_google_feeling_lucky_url( artist, title ) }
	end

	def check_lyrics_page( page_body, artist, title, album=nil, year=nil )
		md = /<title>([^<]+) Lyrics<\/title>/i.match( page_body )
		return false if ( ! md )
		page_title = Strings.normalize( md[1] )
		return page_title.index( Strings.normalize( artist ) ) && page_title.index( Strings.normalize( title ) )
	end

	def parse_lyrics( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )

		lyrics_data = {}

		return lyrics_data if ( ! body.sub!( /^.* lyrics<\/h3>/, "" ) ) # metadata

		["Artist", "Album"].each() do |key|
			if ( md =/#{key}: <b><a href=[^>]+>([^<]+)<\/a><\/b>/.match( body ) )
				lyrics_data[key.downcase()] = md[1].strip()
			end
		end
		["Year", "Title"].each() do |key|
			if ( md =/#{key}: ([^<]+)<(br|\/td)>/.match( body ) )
				lyrics_data[key.downcase()] = md[1].strip()
			end
		end

		return lyrics_data if ( ! body.sub!( /^.*Correct<\/a> ?<\/td> ?<\/tr> ?<\/table> ?(<br>){0,10}/, "" ) )
		return lyrics_data if ( ! body.sub!( /(<b>)?<a href=.*$/, "" ) )

		body.sub!( /(<br ?\/>)?&#91;[^&]+ Lyrics on <\/br>http:\/\/#{site_host()}\/ &#93;/, "" )
		body.sub!( /^.*<\/a>/, "" ) # additional (optional) crap at the beginning
		body.gsub!( /<u>&lt;a[^<]+&lt;\/a&gt;<\/u>/, "" ) # yet more crap

		body.tr_s!( " \n\r\t", " " )
		body.gsub!( /\ ?<br ?\/?> ?/i, "\n" )
		body.strip!()

		lyrics_data["lyrics"] = body

		return lyrics_data

	end

	def build_suggestions_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => build_google_feeling_lucky_url( artist ) }
	end

	def check_suggestions_page( page_body, artist, title, album=nil, year=nil )
		md = /<title>([^<]+) Lyrics<\/title>/i.match( page_body )
		return false if ( ! md )
		page_title = Strings.normalize( md[1] )
		return page_title.index( Strings.normalize( artist ) )
	end

	# returns an array of maps with following keys: url, artist, title
	def parse_suggestions( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )

		suggestions = []

		return suggestions if ( ! body.sub!( /^.*<\/span> <tr> <td width=50% valign=top>/, "" ) )
		return suggestions if ( ! body.sub!( /<br> ?<\/td> ?<\/tr> ?<\/table> ?<br> ?<span style="font-size:14px;">.*$/, "" ) )

		body.split( /<a href=/ ).each() do |entry|
			if ( md = /"(\/lyrics\/[^"]+)" title="[^"]+"> ?([^>]+) lyrics<\/a><br>/.match( entry ) )
				suggestions << { "url"=>"http://#{site_host()}#{md[1]}", "artist"=>artist, "title"=>md[2] }
			end
		end

		return suggestions

	end

end
