#  Copyright (C) 2007 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

if $0 == __FILE__
	puts 'Nothing to see here...'
	puts 'Did you meant to run plugins_tests.rb?'
	exit 1
end

require 'test/unit'

class PluginTests < Test::Unit::TestCase

	def self.build_suite( plugin )
		@@pluginPluginTests = plugin
		eval(
			"class #{plugin.class.name}Tests < PluginTests\n" + \
				"@@plugin = @@pluginPluginTests\n" + \
				"def self.plugin()\n" + \
					"return @@plugin\n" + \
				"end\n" + \
			"end"
		)
		return eval( "#{plugin.class.name}Tests.suite()" )
	end

	def plugin()
		self.class.plugin()
	end

	def check_suggestions( suggestions, title, artist, site_name )
		assert_not_nil( suggestions, "No suggestions for #{title} by #{artist} returned by #{site_name}" )
		assert( ! suggestions.empty?(), "Empty suggestions list for #{title} by #{artist} returned by #{site_name}" )
	end

	def check_lyrics( lyrics, title, artist, site_name )
		assert_not_nil( lyrics, "No lyrics for #{title} by #{artist} returned by #{site_name}" )
		assert( ! lyrics.strip().empty?, "Empty lyrics for #{title} by #{artist} returned by #{site_name}" )
		assert_no_match(
			/<\/?(a|h[1-6]|hr|p|br|body|html|script|!?--) *\/?>*/i, lyrics,
			"Lyrics for #{title} by #{artist} returned by #{site_name} contain unexpected HTML tags"
		)
	end

	def test_lyrics_direct_search()
		puts "\nTesting #{plugin().site_name()} lyrics direct search..."
		self.class.get_test_values( plugin().site_name() ).each() do |test_values|
			artist = test_values['artist']
			album = test_values['album']
			title = test_values['title']
			if ( plugin().build_lyrics_fetch_data( artist, title, album, nil )['url'] )
				puts " - searching lyrics for #{title} by #{artist}"
				ld = plugin().lyrics_direct_search( artist, title, album, nil )
				check_lyrics( ld['lyrics'], title, artist, plugin.site_name() )
			else
				puts " - skipping lyrics search for #{title} by #{artist} (no direct fetch data received)"
				break
			end
		end
	end

	def test_lyrics_from_suggestions()
		puts "\nTesting #{plugin().site_name()} lyrics from suggestions..."
		self.class.get_test_values( plugin().site_name() ).each() do |test_values|
			artist = test_values['artist']
			album = test_values['album']
			title = test_values['title']
			stitle = test_values['stitle']
			puts " - searching suggestions for #{stitle} by #{artist}"
			suggestions = plugin().suggestions( artist, stitle, album, nil )
			check_suggestions( suggestions, stitle, artist, plugin.site_name() )
			puts " - searching lyrics to #{title} by #{artist} in suggestions"
			ld = plugin().lyrics_from_suggestions( artist, title, album, nil, suggestions )
			check_lyrics( ld['lyrics'], title, artist, plugin.site_name() )
		end
	end

	def test_lyrics_full_search()
		puts "\nTesting #{plugin().site_name()} lyrics full search..."
		self.class.get_test_values( plugin().site_name() ).each() do |test_values|
			artist = test_values['artist']
			album = test_values['album']
			title = test_values['title']
			puts " - searching lyrics for #{title} by #{artist}"
			ld = plugin().lyrics_full_search( artist, title, album, nil )
			check_lyrics( ld['lyrics'], title, artist, plugin.site_name() )
		end
	end

	def test_no_lyrics_found()
		puts "\nTesting #{plugin().site_name()} forced retrieval fail..."
		artist = 'Fake Artist'
		album = 'Fake Album'
		title = 'Fake Title'
		puts " - searching lyrics for #{title} by #{artist}"
		ld = plugin().lyrics_full_search( artist, title, album, nil )
		assert_nil( ld['lyrics'], "Unexpected lyrics returned for #{title} by #{artist} by #{plugin.site_name()}" )
	end

	# NOTE: if possible use popular song that are likely to be indexed by google.
	# We are not testing here how good or complete a site is but if the site plugin
	# is behaving as expected. We part from the assumption that every plugin should
	# be able to get lyrics and suggestions for these values.
	def self.get_test_values( site_name )
		ret = []
		if ( site_name == 'Jamendo' )
			ret << { 'artist'=>'Misantropía', 'album'=>'Misantropía', 'title'=>'Hipócrita (Hypocrite)' }
			ret << { 'artist'=>'Inesperado', 'album'=>'Maqueta Inesperada', 'title'=>'Pez Gordo' }
			ret << { 'artist'=>'Tirpunk', 'album'=>'Tirpunk-Újra', 'title'=>'Suicide' }
			ret << { 'artist'=>'Punkamine', 'album'=>'Punkamine', 'title'=>'Gora gerra' }
		elsif ( site_name == 'Not Popular.com' )
			ret << { 'artist'=>'Radiohead', 'album'=>'Kid A', 'title'=>'Optimistic' }
			ret << { 'artist'=>'Nirvana', 'album'=>'Bleach', 'title'=>'About a Girl' }
			ret << { 'artist'=>'System Of A Down', 'album'=>'Toxicity', 'title'=>'Chop Suey!' }
			ret << { 'artist'=>'A Perfect Circle', 'album'=>'Thirteen Step', 'title'=>'The Noose' }
		elsif ( site_name == 'Leos Lyrics' )
			ret << { 'artist'=>'Radiohead', 'album'=>'Kid A', 'title'=>'Optimistic' }
			ret << { 'artist'=>'Nirvana', 'album'=>'Bleach', 'title'=>'About a Girl' }
			ret << { 'artist'=>'System Of A Down', 'album'=>'Toxicity', 'title'=>'Chop Suey' }
			ret << { 'artist'=>'A Perfect Circle', 'album'=>'Thirteen Step', 'title'=>'The Noose' }
		elsif ( site_name == 'Giitaayan' )
			ret << { 'artist'=>'Abhijit, Kavita Subramaniam', 'album'=>'Juhi', 'title'=>ITRANS.to_devanagari( 'a.ndherii raat me.n ham aur tum' ) }
			ret << { 'artist'=>'Poornima, Chorus', 'album'=>'Raajaa Baabu', 'title'=>ITRANS.to_devanagari( 'a aa ii u u uu meraa dil na to.Do' ) }
			ret << { 'artist'=>'Sonu Nigam, Chorus', 'album'=>'Fizaa', 'title'=>ITRANS.to_devanagari('gayaa gayaa dil jaane de' ) }
			ret << { 'artist'=>'Lata, Hemant', 'album'=>'Ek Saal', 'title'=>ITRANS.to_devanagari( 'ulajh gaye do nainaa, dekho' ) }
		elsif ( site_name == 'Baidu MP3' )
			ret << { 'artist'=>'李克勤', 'album'=>'最多李克勤精选','title'=>'在我身边' }
			ret << { 'artist'=>'李克勤', 'album'=>'飞花', 'title'=>'爱不坏' }
			ret << { 'artist'=>'梅艳芳', 'album'=>'时光倒流30年', 'title'=>'歌衫泪影' }
			ret << { 'artist'=>'梅艳芳', 'album'=>'梅艳芳', 'title'=>'似水流年' }
		elsif ( site_name == 'AZ Lyrics' )
			ret << { 'artist'=>'Radiohead', 'album'=>'Kid A', 'title'=>'Optimistic' }
			ret << { 'artist'=>'Pearl Jam', 'album'=>'Ten', 'title'=>'Alive' }
			ret << { 'artist'=>'Massive Attack', 'album'=>'Protection', 'title'=>'Protection' }
			ret << { 'artist'=>'Portishead', 'album'=>'Dummy', 'title'=>'Wandering Star' }
		elsif ( site_name == 'Dark Lyrics' )
			ret << { 'artist'=>'Opeth', 'album'=>'Damnation', 'title'=>'Hope Leaves' }
			ret << { 'artist'=>'Megadeth', 'album'=>'Youthanasia', 'title'=>'À tout le monde' }
			ret << { 'artist'=>'Metallica', 'album'=>'Master of Puppets', 'title'=>'Master of Puppets' }
			ret << { 'artist'=>'Lacuna Coil', 'album'=>'Comalies', 'title'=>'Heaven\'s a Lie' }
		elsif ( site_name == 'Lyrc' )
			ret << { 'artist'=>'Nirvana', 'album'=>'Nevermind', 'title'=>'Smells Like Teen Spirit', 'stitle'=>'Smells Like' }
			ret << { 'artist'=>'Radiohead', 'album'=>'Kid A', 'title'=>'How To Dissapear Completely','stitle'=>'How To Dissapear' }
			ret << { 'artist'=>'Pearl Jam', 'album'=>'Riot Act', 'title'=>'Love Boat Captain', 'stitle'=>'Love Boat' }
			ret << { 'artist'=>'Blur', 'album'=>'13', 'title'=>'No Distance Left To Run', 'stitle'=>'No Distance Left' }
		elsif ( site_name == 'LyricsMania' )
			ret << { 'artist'=>'Nirvana', 'album'=>'Nevermind', 'title'=>'Lounge Act' }
			ret << { 'artist'=>'Radiohead', 'album'=>'Kid A', 'title'=>'Idioteque' }
			ret << { 'artist'=>'Pearl Jam', 'album'=>'Ten', 'title'=>'Porch' }
			ret << { 'artist'=>'The Smashing Pumpkins', 'album'=>'Siamese Dream', 'title'=>'Mayonaise' }
		elsif ( site_name == 'Terra Letras' )
			ret << { 'artist'=>'The Cranberries', 'album'=>'Everybody Else Is Doing It, So Why Can\'t We?', 'title'=>'Linger' }
			ret << { 'artist'=>'Radiohead', 'album'=>'Kid A', 'title'=>'Optimistic' }
			ret << { 'artist'=>'Pearl Jam', 'album'=>'Ten', 'title'=>'Porch' }
			ret << { 'artist'=>'The Smashing Pumpkins', 'album'=>'Siamese Dream', 'title'=>'Mayonaise' }
		elsif ( site_name == 'Sing365' )
			ret << { 'artist'=>'Nirvana', 'album'=>'Nevermind', 'title'=>'Smells Like Teen Spirit' }
			ret << { 'artist'=>'The Cranberries', 'album'=>'Everybody Else Is Doing It, So Why Can\'t We?', 'title'=>'Linger' }
			ret << { 'artist'=>'Pearl Jam', 'album'=>'Ten', 'title'=>'Porch' }
			ret << { 'artist'=>'The Smashing Pumpkins', 'album'=>'Siamese Dream', 'title'=>'Mayonaise' }
		else
			ret << { 'artist'=>'Nirvana', 'album'=>'Nevermind', 'title'=>'Smells Like Teen Spirit' }
			ret << { 'artist'=>'Radiohead', 'album'=>'Kid A', 'title'=>'Optimistic' }
			ret << { 'artist'=>'Pearl Jam', 'album'=>'Ten', 'title'=>'Porch' }
			ret << { 'artist'=>'The Smashing Pumpkins', 'album'=>'Siamese Dream', 'title'=>'Mayonaise' }
		end
		ret.each() { |values| values['stitle'] = values['title'] if ( ! values.include?( 'stitle' ) ) }
		return ret
	end

end
